function workshop (action)
switch action   
case 'start',
   set (gcbf, 'WindowButtonMotionFcn', 'workshop move')
   set (gcbf, 'WindowButtonUpFcn', 'workshop stop')  
   
case 'move',
   currPt=get(gca, 'CurrentPoint');  % get cursor position
   % make sure that can't move cursor outside of axes
   XLm=get(gca, 'XLim');  YLm= get(gca, 'YLim');
   if currPt(1,1) > max(XLm) currPt(1,1)=max(XLm); 
   elseif currPt(1,1) < min(XLm) currPt(1,1)=min(XLm);
   end
   if currPt(1,2) > max(YLm) currPt(1,2)=max(YLm); 
   elseif currPt(1,2) < min(YLm) currPt(1,2)=min(YLm);
   end
   % if considering equil. linearisations, restrict movement to equil locus
   Equil=findobj('Tag', 'Equil Linearisation'); Value=get(Equil, 'Value');
   if Value==1, currPt(1,2)=RhoLocus(currPt(1,1),0.0); end
   % move cursor
   set(gco, 'XData', currPt(1,1))
   set(gco, 'YData', currPt(1,2))
   % update display of position coordinates
   XPosn=findobj('Tag','XPosn'); 
   Txt=strcat('x=',num2str(currPt(1,1)));
   Txt=strcat(Txt,' r='); Txt=strcat(Txt,num2str(currPt(1,2)));
   Txt=strcat(Txt,'  (rho='); Txt=strcat(Txt,num2str(getrho));
   Txt=strcat(Txt,')');
   set(XPosn, 'String', Txt)
   
case 'stop',
   set (gcbf, 'WindowButtonMotionFcn', '')
   set (gcbf, 'WindowButtonUpFcn', '')
   Refresh
   
case 'change system',  
   Refresh
   
case 'Equil Linearisation',
   Equil=findobj('Tag', 'Equil Linearisation');
   Vel=findobj ('Tag', 'Vel Linearisation');
   set(Equil, 'Value', 1)
   set(Vel, 'Value', 0)
   Refresh
   
case 'Vel Linearisation',
   Equil=findobj('Tag', 'Equil Linearisation');
   Vel=findobj ('Tag', 'Vel Linearisation');
   set(Equil, 'Value', 0)
   set(Vel, 'Value', 1)
   Refresh
     
case 'ControlParas'
   % Controller parameters have changed so refresh display
   Refresh
   
case 'Refresh Step Response'
   % update step response window
   StepPlot
   
case 'Change Step Amplitude'
   StepPlot
   
end

%---------------------------------------------------------------------------

function Refresh

if isempty(findobj('Tag','Control Window')) 
   example; % trap error if main display window isn't there
end 

% add equil locus to operating space if not there already
equilline=findobj('Tag','EquilLocus');
if isempty(equilline)
   OpSpace=findobj('Tag', 'Operating Space');
   XLm=get(OpSpace, 'XLim'); 
   r=[XLm(1):(XLm(2)-XLm(1))/25:XLm(2)];
   subplot(OpSpace); equilline=line(r,RhoLocus(r,0.0),'Tag', 'EquilLocus', 'Color', [1 0 0]);
   set (equilline, 'HitTest', 'off'); % prevent line from being selected by user
   % and add some extra contour lines
   line(r,RhoLocus(r,5.0),'LineStyle',':','Color', [1 0 0],'HitTest', 'off');
   line(r,RhoLocus(r,-5.0),'LineStyle',':','Color', [1 0 0],'HitTest', 'off');
   line(r,RhoLocus(r,10.0),'LineStyle',':','Color', [1 0 0],'HitTest', 'off');
   line(r,RhoLocus(r,-10.0),'LineStyle',':','Color', [1 0 0],'HitTest', 'off');
end

% get system 
ChooseSystem=findobj('Tag', 'Choose System');   
Value=get(ChooseSystem, 'Value'); String=get(ChooseSystem,'String');
System=String{Value}; 

% display window with controller parameters if appropriate
mainwin=findobj('Tag','Control Window');
PIwin=findobj('Tag','Linear PI Paras');
Invwin=findobj('Tag', 'Inverse Paras');
schedPIwin=findobj('Tag', 'Scheduled PI Paras');
switch System
case {'Linear PI Controller','Plant+Linear PI'}
   if ~isempty(Invwin) set(Invwin, 'Visible', 'off'); end
   if ~isempty(schedPIwin) set(schedPIwin, 'Visible','off'); end
   if isempty(PIwin)  
      PIwin=linearpiparas2; 
      Posn=get(mainwin, 'Position');
      NewPosn=get(PIwin, 'Position');
      NewPosn(1)=Posn(1)+Posn(3)*0.75; NewPosn(2)=Posn(2)-Posn(4)*0.05;
      set(PIwin, 'Position', NewPosn);
   end
   set(PIwin, 'Visible','on');
   K0val=findobj('Tag', 'KpSlider'); K0=get(K0val,'Value'); set(findobj('Tag','LinearKp'),'String',num2str(K0)); 
   K1val=findobj('Tag', 'KiSlider'); K1=get(K1val,'Value'); set(findobj('Tag','LinearKi'),'String',num2str(K1));
   conwin=PIwin;
case {'Inverse Controller', 'Plant+Inverse'}
   if ~isempty(PIwin) set(PIwin, 'Visible','off'); end
   if ~isempty(schedPIwin) set(schedPIwin, 'Visible','off'); end
   if isempty(Invwin)  
      Invwin=invparas; 
      Posn=get(mainwin, 'Position');
      NewPosn=get(Invwin, 'Position');
      NewPosn(1)=Posn(1)+Posn(3)*0.75; NewPosn(2)=Posn(2)-Posn(4)*0.05;
      set(Invwin, 'Position', NewPosn);
   end
   set(Invwin, 'Visible','on');      
   epsdval=findobj('Tag', 'epsd slider'); epsd=get(epsdval,'Value'); set(findobj('Tag','epsd'),'String',num2str(epsd)); 
   lamval=findobj('Tag', 'lambda slider'); lambda=get(lamval, 'Value'); set(findobj('Tag', 'lambda'),'String',num2str(lambda)); 
   conwin=Invwin;
case {'Scheduled PI','Plant+Scheduled PI'}   
   if ~isempty(Invwin) set(Invwin, 'Visible', 'off'); end
   if ~isempty(PIwin) set(PIwin, 'Visible','off'); end
   if isempty(schedPIwin)  
      schedPIwin=scheduledpiparas; 
      Posn=get(mainwin, 'Position');
      NewPosn=get(schedPIwin, 'Position');
      NewPosn(1)=Posn(1)+Posn(3)*0.75; NewPosn(2)=Posn(2)-Posn(4)*0.05;
      set(schedPIwin, 'Position', NewPosn);
   end
   set(schedPIwin, 'Visible','on');            
   K0val=findobj('Tag', 'Kp0Slider'); K0=get(K0val,'Value'); set(findobj('Tag','Kp0'),'String',num2str(K0)); 
   K1val=findobj('Tag', 'Ki0Slider'); K1=get(K1val,'Value'); set(findobj('Tag','Ki0'),'String',num2str(K1));
   K0val=findobj('Tag', 'Kp1Slider'); K0=get(K0val,'Value'); set(findobj('Tag','Kp1'),'String',num2str(K0)); 
   K1val=findobj('Tag', 'Ki1Slider'); K1=get(K1val,'Value'); set(findobj('Tag','Ki1'),'String',num2str(K1));
   conwin=schedPIwin;
otherwise
   if ~isempty(Invwin) set(Invwin, 'Visible', 'off'); end
   if ~isempty(PIwin) set(PIwin, 'Visible','off'); end
   if ~isempty(schedPIwin) set(schedPIwin, 'Visible','off'); end
   conwin=[];   
end

% update step response plotting options
switch System
case {'Plant','Approx Plant','Plant+Linear PI','Plant+Inverse','Plant+Scheduled PI'}
   cmat={'Output', 'Input', 'Rho', 'Plant Input'};
otherwise
   cmat={'Output', 'Input'};
end
VarList=findobj('Tag','Output Variable'); 
if ~isempty(VarList) set(VarList,'String',cmat); end

% now update Bode plot
BodePlot

% and make sure than control parameter window is left selected
if ~isempty(conwin) figure(conwin); end

%---------------------------------------------------------------------------

function BodePlot

if isempty(findobj('Tag','Control Window')) return; end % trap error if display window isn't there

% system to be plotted
ChooseSystem=findobj('Tag', 'Choose System');   
Value=get(ChooseSystem, 'Value'); String=get(ChooseSystem,'String');
System=String{Value}; 

% value of rho
rho=getrho;

% get linearised dynamics in state-space form
switch System
case 'Plant',
   [A,B,C,D]=Plant(rho);
case 'Linear PI Controller',
   [A,B,C,D]=LinearPI(rho);
case 'Plant+Linear PI',
   [pA,pB,pC,pD]=Plant(rho);
   [cA,cB,cC,cD]=LinearPI(rho); 
   A=[pA pB*cC; -cB*pC cA-cB*pD*cC]; %assuming here that SISO and cD=0
   B=[0*pB;cB]; C=[pC pD*cC]; D=[0]; % closed loop state space matrices
case 'Inverse Controller',
   [A,B,C,D]=Inverse(rho);   
case 'Plant+Inverse',
   [pA,pB,pC,pD]=Plant(rho);
   [cA,cB,cC,cD]=Inverse(rho); 
   A=[pA-pB*cD*pC pB*cC; -cB*pC cA]; %assuming here that SISO and pD=0
   B=[pB*cD;cB]; C=[pC 0*cC]; D=[0]; % closed-loop state space matrices 
case 'Approx Plant'
   [A,B,C,D]=ApproxPlant(rho);   
case 'Approx Error'
   [pA,pB,pC,pD]=Plant(rho);
   [cA,cB,cC,cD]=ApproxPlant(rho);    
   % subtract output of approx and from output of exact plant
   A=[pA 0;0 cA]; B=[pB; cB]; C=[pC -cC]; D=[pD-cD]; 
case 'Scheduled PI',
   [A,B,C,D]=ScheduledPI(rho);
case 'Plant+Scheduled PI',      
   [pA,pB,pC,pD]=Plant(rho);
   [cA,cB,cC,cD]=ScheduledPI(rho); 
   A=[pA pB*cC; -cB*pC cA-cB*pD*cC]; %assuming here that SISO and cD=0
   B=[0*pB;cB]; C=[pC pD*cC]; D=[0]; % closed loop state space matrices
end

%Now calculate transfer function
win=findobj('Tag','Control Window'); figure(win);
magaxes=findobj('Tag', 'MagAxes'); subplot(magaxes) % select appropriate axes on screen
XLm=get(magaxes,'XLim');
w=logspace(log10(XLm(1)),log10(XLm(2)));  s=sqrt(-1)*w; % frequency range
  % Reduce to Hessenberg form then directly evaluate frequency response.
  [ny,nu] = size(D); [nx,na] = size(A); nw = max(size(s));
  % Reduce A to Hessenburg form
  [p,A] = hess(A);
  % Apply similarity transformations from Hessenberg
  % reduction to B and C:
  if nx>0,
    B = p' * B;
    C = C * p;
    g = ltifr(A,B,s(:));
    g = (C * g + diag(D) * ones(ny,nw)).';
  else
    g = (diag(D) * ones(ny,nw)).';
 end
 % convert to polar form with mag in dB's
  mag1=abs(g);
  mag=20*log10(mag1); 
  phase = (180./pi)*unwrap(atan2(imag(g),real(g)));
 %Try to correct phase anomaly for plants with integrators
 %by adding multiples of -360 degrees.
 if abs(det(A)) < eps
	 if phase(1) > 0 & nintgs > 0
	     phase = phase - 360;
	 end
 end

% and display it
subplot(magaxes) % select appropriate axes on screen
magline=findobj('Tag', 'MagLine');
if isempty(magline) %nothing plotted previously so create new curve
   magline=line(w,mag,'Tag', 'MagLine', 'color',[1 0 0]);
else
   set(magline,'Xdata',w,'Ydata',mag, 'color',[1 0 0]);
end
phaseaxes=findobj('Tag', 'PhaseAxes'); subplot(phaseaxes) % select appropriate axes on screen
phaseline=findobj('Tag', 'PhaseLine');
if isempty(phaseline)
   phaseline=line(w,phase, 'Tag', 'PhaseLine', 'color',[1 0 0]);
else
   set(phaseline,'Xdata',w,'Ydata',phase, 'color',[1 0 0]);
end
% calculate gain and phase margins
%if max(mag1)> 1 & min(mag1) < 1
%   i=find(mag<1); i=min(i);
%   wc=interp1(mag1(i-1:i),w(i-1:i), 1.0); % cross-over frequency
%   pm=180.0+interp1(w,phase,wc); % phase margin
%else
%   if max(mag1) <1
%      wc=NaN; pm=inf;
%   else
%      wc=NaN; pm=NaN;
%   end
%end
%if max(phase)> -180 & min(phase) < -180
%   i=find(phase<-180); i=min(i);
%   wp=interp1(phase(i-1:i),w(i-1:i), -180); % -180 deg. frequency
%   gm=interp1(w,mag,wp); % gain margin
%else
%   if min(phase) > -180
%      gm=inf;
%   else
%      gm=NaN;
%   end
%end
% update display of stability margins
%Txt=strcat('wc=',num2str(wc));
%Txt=strcat(Txt,' gm='); Txt=strcat(Txt,num2str(gm));
%Txt=strcat(Txt,'  pm='); Txt=strcat(Txt,num2str(pm));
%StabM=findobj('Tag','Stability Margins'); 
%set(StabM, 'String', Txt); % stability margin display disabled !

%---------------------------------------------------------------------------

function StepPlot
% Step response of nonlinear system
global r K0 K1 epsd lambda K00 K10 K01 K11 % input, controller parameters

if isempty(findobj('Tag','Control Window')) Refresh; end 

% display step response window if not already there
win=findobj('Tag', 'Step Window');
if isempty(win)
   win=stepresponse;
   mainwin=findobj('Tag','Control Window');
   if isempty(mainwin) mainwin=gui1; end
   Posn=get(mainwin, 'Position');
   NewPosn=get(win, 'Position');
   NewPosn(1)=Posn(1)+Posn(3)*0.1; NewPosn(2)=Posn(2)-Posn(4)/4;
   set(win, 'Position', NewPosn);      
end

% get system 
ChooseSystem=findobj('Tag', 'Choose System');   
Value=get(ChooseSystem, 'Value'); String=get(ChooseSystem,'String');
System=String{Value}; 

% get function describing nonlinear dynamics
switch System
case 'Plant',
   funct='plantfn';
case 'Linear PI Controller',
   K0val=findobj('Tag', 'KpSlider'); K0=get(K0val,'Value');
   K1val=findobj('Tag', 'KiSlider'); K1=get(K1val,'Value');
   funct='pifn';
case 'Plant+Linear PI',
   K0val=findobj('Tag', 'KpSlider'); K0=get(K0val,'Value');
   K1val=findobj('Tag', 'KiSlider'); K1=get(K1val,'Value');
   funct='plantpifn';   
case 'Inverse Controller',
   % not possible to calculate step response of controller alone since uses rho from plant
   win=findobj('Tag','Step Response Plot'); set(win,'Visible','off');
   curve=findobj('Tag','Step Response Curve'); set(curve, 'Visible', 'off');
   win=findobj('Tag','Err Message2'); set(win, 'Visible','off');
   win=findobj('Tag','Err Message'); set(win, 'Visible','on');
   return
case 'Plant+Inverse',
   epsdval=findobj('Tag', 'epsd'); epsd=str2num(get(epsdval,'String'));
   lamval=findobj('Tag', 'lambda'); lambda=str2num(get(lamval,'String'));
   funct='plantinversefn';
case 'Approx Plant'
   % step response not implemented
   win=findobj('Tag','Step Response Plot'); set(win,'Visible','off');
   curve=findobj('Tag','Step Response Curve'); set(curve, 'Visible', 'off');
   win=findobj('Tag','Err Message'); set(win, 'Visible','off');
   win=findobj('Tag','Err Message2'); set(win, 'Visible','on');
   return
case 'Approx Error'
   % step response not implemented
   win=findobj('Tag','Step Response Plot'); set(win,'Visible','off');
   curve=findobj('Tag','Step Response Curve'); set(curve, 'Visible', 'off');
   win=findobj('Tag','Err Message'); set(win, 'Visible','off');
   win=findobj('Tag','Err Message2'); set(win, 'Visible','on');
   return
case 'Scheduled PI',
   % not possible to calculate step response of controller alone since uses rho from plant
   win=findobj('Tag','Step Response Plot'); set(win,'Visible','off');
   curve=findobj('Tag','Step Response Curve'); set(curve, 'Visible', 'off');
   win=findobj('Tag','Err Message2'); set(win, 'Visible','off');
   win=findobj('Tag','Err Message'); set(win, 'Visible','on');
   return
case 'Plant+Scheduled PI',      
   K0val=findobj('Tag', 'Kp0'); K00=str2num(get(K0val,'String'));
   K1val=findobj('Tag', 'Ki0'); K10=str2num(get(K1val,'String'));
   K0val=findobj('Tag', 'Kp1'); K01=str2num(get(K0val,'String')); 
   K1val=findobj('Tag', 'Ki1'); K11=str2num(get(K1val,'String')); 
   funct='plantschedpifn';   
end

% amplitude of step to be applied
AmpVal=findobj('Tag', 'Step Amplitude'); Amp=get(AmpVal,'Value');
set(findobj('Tag','Step Amplitude Text'),'String',num2str(Amp));
r=Amp; % define step input

% now solve differential equation
[t,y]=ode15s(funct); %y(:,1) is output and y(:,2) is plant input (for closed-loop systems)

% and plot response ...
win=findobj('Tag','Err Message'); set(win, 'Visible','off');
win=findobj('Tag','Err Message2'); set(win, 'Visible','off');
win=findobj('Tag','Step Response Plot'); subplot (win); set(win,'Visible','on');

% determine which quantity to display
VarList=findobj('Tag','Output Variable');
Value=get(VarList, 'Value'); String=get(VarList,'String');
switch String{Value}
case 'Output'
   yy=y(:,1);
case 'Input'
   yy=r*ones(size(t));
case 'Rho'
   % this option is only appropriate when have plant
   yy=CalcRho(y(:,1),y(:,2));
case 'Plant Input'
   % this option is only appropriate when have plant
   yy=y(:,2);
end

% scale y-axis to fit response
if max(abs(yy)) < 10 
   k=1;
else
   k=10;
end
set(win,'YLim',k*[(floor(min(yy)/k)) ceil(max([yy])/k+1)]); 
% and display step response curve
curve=findobj('Tag','Step Response Curve');
if isempty(curve)
   curve=line(t,yy,'Tag','Step Response Curve', 'Visible', 'on');
else
   set(curve,'XData',t,'YData',yy, 'Visible', 'on');
end


%---------------------------------------------------------------------------

function rho=getrho
% Calulates value of rho from position of pointer in operating space
Cursor=findobj('Tag', 'Pos'); % get handle of pointer
r=get(Cursor, 'XData'); x=get(Cursor, 'YData'); % position in operating space
rho=CalcRho(x,r);

%---------------------------------------------------------------------------

function x=RhoLocus(r,rho)
%calculate state corresponding to input r (vector of values)and a particular value of rho
x=(r-rho)/10;

%---------------------------------------------------------------------------

function rho=CalcRho(x,r)
%calculate rho
rho=r-10*x;

%---------------------------------------------------------------------------
function [A,B,C,D]=Plant(rho)
% Plant linearisation
dG = 1.01-tanh(rho).^2;
A=-10*dG; B=dG; C=1; D=[0];
  
%---------------------------------------------------------------------------

function [A,B,C,D]=LinearPI(rho)
% Linear PI Controller linearisation
K0val=findobj('Tag', 'LinearKp'); K0=str2num(get(K0val,'String'));
K1val=findobj('Tag', 'LinearKi'); K1=str2num(get(K1val,'String'));
A=[-50 0;1 0]; B=[50;0]; C=[K0 K1]; D=[0];
  
%---------------------------------------------------------------------------

function [A,B,C,D]=Inverse(rho)
% Inverse Controller (with integral action) linearisation
epsdval=findobj('Tag', 'epsd'); epsd=str2num(get(epsdval,'String'));
lamval=findobj('Tag', 'lambda'); lambda=str2num(get(lamval,'String'));

dG = 1.01-tanh(rho).^2;
A=[0 0;-dG/epsd -dG*(10+1/epsd)]; B=[lambda;0]; C=[1/epsd 1/epsd]; D=[0];

%---------------------------------------------------------------------------

function [A,B,C,D]=ApproxPlant(rho)
% linearisation of blended multiple model approximation to plant

% two local models are used ...
dG0 = 1.01-tanh(0).^2;
A0=-10*dG0; B0=dG0; C0=1; D0=[0];%plant vel based lin on the equil locus
dG1 = 1.01-tanh(5).^2;
A1=-10*dG1; B1=dG1; C1=1; D1=[0]; % plant vel based lin far from equil locus

% plus piecewise cosine blending functions
% weight 0 - cosine weighting centred on origin 
%x=0.35*rho*pi;
%if abs(x) <=pi      weight0 = (cos(x)+1)/2;
%               else weight0 = 0.0;           end 
% gaussian weighting centred on origin - better fit than cosine weighting
x=0.8*rho;
weight=exp(-x.^2);
weight1=(1-weight).^2; weight0=1-weight1;

% model
A=weight0*A0+weight1*A1; B=weight0*B0+weight1*B1;
C=weight0*C0+weight1*C1; D=weight0*D0+weight1*D1;

%---------------------------------------------------------------------------

function [A,B,C,D]=ScheduledPI(rho)
% Scheduled PI Controller linearisation

% local controller 0
K0val=findobj('Tag', 'Kp0'); K0=str2num(get(K0val,'String'));
K1val=findobj('Tag', 'Ki0'); K1=str2num(get(K1val,'String'));
A0=[-50 0;1 0]; B0=[50;0]; C0=[K0 K1]; D0=[0];
%local controller 1
K0val=findobj('Tag', 'Kp1'); K0=str2num(get(K0val,'String'));
K1val=findobj('Tag', 'Ki1'); K1=str2num(get(K1val,'String'));
A1=[-50 0;1 0]; B1=[50;0]; C1=[K0 K1]; D1=[0];

% gaussian-like weighting about origin 
x=0.8*rho; weight=exp(-x.^2);
weight1=(1-weight).^2; weight0=1-weight1;


% model
A=weight0*A0+weight1*A1; B=weight0*B0+weight1*B1;
C=weight0*C0+weight1*C1; D=weight0*D0+weight1*D1;

%---------------------------------------------------------------------------

